/*#############################################################
File Name       : ex1_HC06_setup.c
Author          : Grant Phillips
Date Modified   : 11/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F429I-Discovery

Description     : Example program that uses the USART1 module to 
									establish a serial link via a HC-06 Bluetooth
									module with another device.  This example only
									shows how to make changes to the HC-06's setup
									using the HC06_F429I.c library.  The following
									functions will be demonstrated:
									HC06_Test
									HC06_SetBaud
									HC06_SetName
									HC06_SetPin
									HC06_ClearRxBuffer
									HC06_PutStr
									If the setup was successfull, the STM32F429I will
									send a continueous count as a string to another 
									device that has established a SPP connection
									with it.
									
									The on-board LCD is used to display debug messages
									
									************** SPECIAL NOTE: *****************
									The default settings for the HC-06 module is:
									Baud: 			9600
									Name:				linvor
									Pin:				1234
									If you want to change any of this, run the
									ex1_HC06_setup.c example first and make the
									necessary changes.  The rest of the example
									programs asumes that the HC-06 is running at 
									115200 baud, so at least change the buad rate
									in the setup program.  The new settings will
									be saved, even when power is removed.      

Requirements    : * STM32F429I-Discovery Board
									* Make sure the STM32F429I_Defaults.c file is 
										in the same folder than the project
									* Make sure the HC06_F429I.c file is in the 
                    same folder than the project
              
Circuit         : * HC06 Bluetooth module is connected as follows:
										TXD			-		PA10
										RXD			-		PA9
										GND			-		GND
										VCC			-		3.3V

See the STM32F429I-Discovery User Manual (UM1670) for the block 
diagram of the STM32F429ZIT6 processor (p11), a summary of the 
GPIO connections and free IO pins (p19-25) and the schematic 
diagram (p27-33).
##############################################################*/

#include "STM32F429I_Defaults.c"																//library for the defaults of a blank project in Keil uVision
#include "HC06_F429I.c"																					//for the HC06 functions on the STM32F429I-Discovery

int main(void)
{
	uint8_t count=0;																							//8-bit integer variable to hold the count value (0-255)
	char buf[40];
	uint8_t rcv;
	char lcdbuf[20];										//declare a string variable of 20 chars 
		
	/* Set the SysTick Interrupt to occur every 1ms) */
	if (SysTick_Config(SystemCoreClock / 1000))
  { 
    while (1);																									//will end up in this infinite loop if there was an error with Systick_Config
  }
	
	/* Initialize the LCD */
  LCD_Init();														//initialize LCD
  LCD_LayerInit();											//initialize LCD layer settings
  LTDC_Cmd(ENABLE);											//enable the TFT LCD Controller
  LCD_SetLayer(LCD_FOREGROUND_LAYER);		//Set LCD foreground layer
	
	
	/* Clear the LCD */ 
  LCD_Clear(LCD_COLOR_BLACK);							//clear the LCD with YELLOW pixels
	
	LCD_SetFont(&Font8x8);
	LCD_SetTextColor(LCD_COLOR_WHITE);					//set forecolor for text
	LCD_SetBackColor(LCD_COLOR_BLACK);					//set backcolor for text
	
	
	// ************ NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB *************
	//The HC-06 is shipped with a baud rate of 9600.  If you changed it 
	//with this setup program, you need to change HC06_Init() to this baud speed too.
	//Also keep in mind that all the sample code works with a baud of 115200.  So you can
	//either change the HC-06 baud with this setup code (see the "Set Baud" code further down)
	//or you can change the HC06_Init(115200) to HC06_Init(9600) in all the sample code.
	// ************ NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB NB *************
	HC06_Init(115200);																								//initialize the HC-06 module at 9600 baud
	
	sprintf(lcdbuf, "Debug messages on LCD:");
	LCD_DisplayStringLine(LINE(0), (uint8_t*)lcdbuf);
  Delay(1000);
	
	//Test communications with HC-06. rcv = 0 if ok
	sprintf(lcdbuf, "Test Communications...");
	LCD_DisplayStringLine(LINE(1), (uint8_t*)lcdbuf);
	rcv = HC06_Test();												
	if(rcv == 0)
		sprintf(lcdbuf, "OK");
	else
		sprintf(lcdbuf, "Err %d", rcv);
	LCD_DisplayStringLine(LINE(2), (uint8_t*)lcdbuf);
	
	//Set the default Bluetooth name for this STM32F4 
	//as it would be seen by other devices - MAX 13 characters
	sprintf(lcdbuf, "Set Name...");
	LCD_DisplayStringLine(LINE(3), (uint8_t*)lcdbuf);	
	rcv = HC06_SetName("F4_GRANT");
	if(rcv == 0)
		sprintf(lcdbuf, "OK");
	else
		sprintf(lcdbuf, "Err %d", rcv);
	LCD_DisplayStringLine(LINE(4), (uint8_t*)lcdbuf);
	
	//Set the default PIN for the HC-06 on this STM32F4 
	//which other devices will use to pair with this device
	sprintf(lcdbuf, "Set Pin...");
	LCD_DisplayStringLine(LINE(5), (uint8_t*)lcdbuf);
	rcv = HC06_SetPin("1234");
	if(rcv == 0)
		sprintf(lcdbuf, "OK");
	else
		sprintf(lcdbuf, "Err %d", rcv);
	LCD_DisplayStringLine(LINE(6), (uint8_t*)lcdbuf);
	
	//Set Baud rate.  If you change this, make sure next time you run this code 
	//you change the HC06_Init(115200) at the beginning to the new Baud rate
	//to effectively communicate with the module.  This new Baud rate is also
	//what it should be set to on the other device
	//
	//This section will be commented out, so if you need to change the 
	//baud rate, uncomment the next 6 lines
	
//	sprintf(lcdbuf, "Set Baud...");
//	LCD_DisplayStringLine(LINE(7), (uint8_t*)lcdbuf);	
//	rcv = HC06_SetBaud(115200);
//	if(rcv == 0)
//		sprintf(lcdbuf, "OK\n");
//	else
//		sprintf(lcdbuf, "Err %d\n", rcv);
//	LCD_DisplayStringLine(LINE(8), (uint8_t*)lcdbuf);
	
	
	//Clear the Receive buffer from any unwanted characters after the setup was done
	HC06_ClearRxBuffer();
	
	sprintf(lcdbuf, "Counting over bluetooth...");
	LCD_DisplayStringLine(LINE(9), (uint8_t*)lcdbuf);
	
	/* Main program loop */
  while (1)
  {
		sprintf(buf, "%2X\n", count); 															//print a integer value (in HEX) and newline to the buf string
		HC06_PutStr(buf);																						//write this string via the HC-06 Bluetooth module as a serial string
		
		count = count + 1;
    Delay(250);
	}
}
